﻿
namespace Perception.RPC.Demo
{
     public class FrequencyCalculator
     {
          private enum SearchState
          {
               FirstOutsideBand,
               Rising,
               Above55,
               Falling,
               Below45
          }

          public static double GetSignalFrequency(float[] aData, double XStep)
          {
               if (aData.Length == 0)
                    return 0;

               double Xo = 0;
               double X = Xo;
               float dPreviousValue = 0.0f;
               double dPreviousX = 0.0f;
               double dSumPeriod = 0.0;
               double dSumPeriod2 = 0.0;

               // Get Min and Max values
               float dMax = float.MinValue;
               float dMin = float.MaxValue;
               for (int i = 0; i < aData.Length; i++)
               {
                    if (aData[i] > dMax) dMax = aData[i];
                    if (aData[i] < dMin) dMin = aData[i];
               }

               SearchState sState = SearchState.FirstOutsideBand;

               //
               // To eliminate noise we use a 5% hysterese band around 0
               //
               float d45 = dMin + 0.45f * (dMax - dMin);
               float d50 = dMin + 0.50f * (dMax - dMin);
               float d55 = dMin + 0.55f * (dMax - dMin);

               bool bFoundFirst = false;
               bool bCountOnRising = false;
               double dTime = 0.0;
               double dPreviousCrossing = 0.0;
               double dPeriod = 0.0;
               long lCycles = 0;

               if (aData[0] == d50)
               {
                    // Now we have to find a sample outside the Hysteresis band to be able to
                    // set the bCountOnRising value
                    sState = SearchState.FirstOutsideBand;
               }
               else
               {
                    bCountOnRising = (aData[0] < d50);

                    if (bCountOnRising)
                         sState = SearchState.Rising;
                    else
                         sState = SearchState.Falling;
               }

               //
               // Initialize the value of previous sample 
               // 
               dPreviousValue = aData[0];
               dPreviousX = X;

               for (int i = 1; i < aData.Length; i++)
               {
                    X = Xo + (i * XStep);
                    if (sState == SearchState.FirstOutsideBand)
                    {
                         if (aData[i] > d55)
                         {
                              bCountOnRising = true;
                              sState = SearchState.Rising;
                         }
                         else if (aData[i] < d45)
                         {
                              bCountOnRising = false;
                              sState = SearchState.Falling;
                         }
                    }

                    if (sState == SearchState.Rising)
                    {
                         if (aData[i] > d50)
                         {
                              sState = SearchState.Above55;
                              if (bCountOnRising)
                              {
                                   dTime = dPreviousX + (X - dPreviousX) * (d50 - dPreviousValue) / (aData[i] - dPreviousValue);
                                   if (!bFoundFirst)
                                   {
                                        bFoundFirst = true;
                                        dPreviousCrossing = dTime;
                                   }
                                   else
                                   {
                                        dPeriod = dTime - dPreviousCrossing;
                                        dSumPeriod += dPeriod;
                                        dSumPeriod2 += dPeriod * dPeriod;
                                        dPreviousCrossing = dTime;
                                        lCycles++;
                                   }
                              }
                         }
                    }

                    if (sState == SearchState.Above55)
                    {
                         if (aData[i] > d55)
                              sState = SearchState.Falling;
                    }

                    if (sState == SearchState.Falling)
                    {
                         if (aData[i] < d50)
                         {
                              sState = SearchState.Below45;
                              if (!bCountOnRising)
                              {
                                   dTime = dPreviousX + (X - dPreviousX) * (d50 - dPreviousValue) / (aData[i] - dPreviousValue);
                                   if (!bFoundFirst)
                                   {
                                        bFoundFirst = true;
                                        dPreviousCrossing = dTime;
                                   }
                                   else
                                   {
                                        dPeriod = dTime - dPreviousCrossing;
                                        dSumPeriod += dPeriod;
                                        dSumPeriod2 += dPeriod * dPeriod;
                                        dPreviousCrossing = dTime;
                                        lCycles++;
                                   }
                              }
                         }
                    }

                    if (sState == SearchState.Below45)
                    {
                         if (aData[i] < d45)
                              sState = SearchState.Rising;
                    }

                    dPreviousValue = aData[i];
                    dPreviousX = X;


               }

               if (lCycles == 0)
                    return 0;
               return GetAverageFrequency(dSumPeriod, lCycles);

          }

          private static double GetAverageFrequency(double dSumPeriod, long lCycles)
          {
               double dPeriod = GetAverageOfPeriods(dSumPeriod, lCycles);
               if (double.IsNaN(dPeriod))
                    return double.NaN;
               else
                    return 1.0 / dPeriod;
          }

          private static double GetAverageOfPeriods(double dSumPeriod, long lCycles)
          {
               double dRet;

               if (lCycles < 1)
                    dRet = double.NaN;
               else
               {
                    dRet = (dSumPeriod / lCycles);
                    if (dRet < 1E-200) // Gee, this is a small period !!
                         dRet = double.NaN;
               }
               return dRet;
          }

     }
}
