﻿using System.ComponentModel;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Windows.Forms;

namespace Perception.RPC.Demo
{
     public enum eGradientPanelStyle
     {
          Horizontal,
          Vertical,
          ForwardDiagonal,
          BackwardDiagonal,
          Eliptical
     }

     public partial class Gradientpanel : Panel
     {
          private Color m_PageStartColor = Color.Blue;
          private Color m_PageEndColor = Color.White;
          private eGradientPanelStyle m_GradientStyle = eGradientPanelStyle.Eliptical;


          /// <summary>
          /// The style of the 3D border.
          /// </summary>
          protected Border3DStyle m_Border3DStyle = Border3DStyle.Etched;

          /// <summary>
          /// Gets or sets the style of the 3D border.
          /// </summary>
          [Browsable(true), Category("Appearance Extended"), Description("Specifies the 3D border")]
          public Border3DStyle Border3DStyle
          {
               get
               {
                    return m_Border3DStyle;
               }
               set
               {
                    m_Border3DStyle = value;
                    Invalidate();
               }
          }

          [Browsable(true), Category("Appearance Extended"), Description("Gradient Start color")]
          public Color PageStartColor
          {
               get { return m_PageStartColor; }
               set { m_PageStartColor = value; PaintGradient(); }
          }

          [Browsable(true), Category("Appearance Extended"), Description("Gradient End color")]
          public Color PageEndColor
          {
               get { return m_PageEndColor; }
               set { m_PageEndColor = value; PaintGradient(); }
          }

          [Browsable(true), Category("Appearance Extended"), Description("Gradient style")]
          public eGradientPanelStyle GradientStyle
          {
               get { return m_GradientStyle; }
               set { m_GradientStyle = value; PaintGradient(); }
          }

          public Gradientpanel()
          {
               InitializeComponent();
               PaintGradient();
          }

          public Gradientpanel(IContainer container)
          {
               container.Add(this);

               InitializeComponent();
               PaintGradient();

          }

          protected override void OnPaint(PaintEventArgs pe)
          {
               BorderStyle = BorderStyle.None;
               DoubleBuffered = true;

               base.OnPaint(pe);
               ControlPaint.DrawBorder3D(pe.Graphics, new Rectangle(0, 0, Width - 1, Height - 1), m_Border3DStyle);
          }

          private void PaintGradient()
          {
               if ((this.Width < 0) || (this.Height < 0)) return;
               Bitmap memoryBitmap = new Bitmap(this.Width, this.Height);
               Graphics memGraphics = Graphics.FromImage(memoryBitmap);

               if (m_GradientStyle == eGradientPanelStyle.Eliptical)
               {
                    GraphicsPath GraphPath = new GraphicsPath();
                    GraphPath.AddEllipse(-this.Width / 2, -this.Height / 2, 2 * this.Width, 2 * this.Height);
                    GraphPath.CloseFigure();
                    PathGradientBrush PathGradBrush = new PathGradientBrush(GraphPath);
                    PathGradBrush.CenterColor = m_PageStartColor;
                    PathGradBrush.SurroundColors = new Color[] { m_PageEndColor };
                    memGraphics.FillPath(PathGradBrush, GraphPath);
                    GraphPath.Dispose();
                    PathGradBrush.Dispose();
               }
               else
               {
                    Rectangle rectBrush = new Rectangle(0, 0, this.Width, this.Height);

                    LinearGradientMode lgm = LinearGradientMode.Vertical;
                    if (m_GradientStyle == eGradientPanelStyle.Horizontal)
                         lgm = LinearGradientMode.Horizontal;
                    else if (m_GradientStyle == eGradientPanelStyle.BackwardDiagonal)
                         lgm = LinearGradientMode.BackwardDiagonal;
                    else if (m_GradientStyle == eGradientPanelStyle.ForwardDiagonal)
                         lgm = LinearGradientMode.ForwardDiagonal;

                    LinearGradientBrush gradBrush = new LinearGradientBrush(rectBrush, m_PageStartColor, m_PageEndColor, lgm);

                    memGraphics.FillRectangle(gradBrush, new Rectangle(0, 0, this.Width, this.Height));
                    gradBrush.Dispose();
               }
               this.BackgroundImage = memoryBitmap;
               this.BackgroundImageLayout = ImageLayout.Stretch;
          }
     }
}
